import React from 'react'
import { Button } from '@pubsweet/ui'
import styled from 'styled-components'
import { reduxForm } from 'redux-form'
import { th } from '@pubsweet/ui-toolkit'

import { ContextualBox, Row, Text } from '../'
import { ManuscriptFiles, DetailsAndAuthors, ResponseToReviewer } from './'

const SubmitRevision = ({
  journal,
  addFile,
  fragment,
  addAuthor,
  deleteFile,
  collection,
  changeForm,
  isFetching,
  currentUser,
  previewFile,
  hasFormError,
  deleteAuthor,
  getSignedUrl,
  handleSubmit,
  responseFile,
  downloadFile,
  fetchingError,
  addResponseFile,
  deleteResponseFile,
  onAuthorEdit,
  isEditingAuthor,
}) => (
  <ContextualBox highlight label="Submit Revision" mb={2}>
    <Root>
      <DetailsAndAuthors
        addAuthor={addAuthor}
        changeForm={changeForm}
        collection={collection}
        deleteAuthor={deleteAuthor}
        fragment={fragment}
        isAuthorEdit={isEditingAuthor}
        manuscriptTypes={journal.manuscriptTypes}
        onAuthorEdit={onAuthorEdit}
      />

      <ManuscriptFiles
        changeForm={changeForm}
        collection={collection}
        deleteFile={deleteFile}
        downloadFile={downloadFile}
        formName="revision"
        fragment={fragment}
        getSignedUrl={getSignedUrl}
        previewFile={previewFile}
        token={currentUser.token}
        uploadFile={addFile}
      />

      <ResponseToReviewer
        file={responseFile}
        getSignedUrl={getSignedUrl}
        isFetching={isFetching}
        onDelete={deleteResponseFile}
        onUpload={addResponseFile}
        token={currentUser.token}
      />

      <Row>
        {hasFormError && (
          <Text align="center" error mt={1}>
            There are some missing required fields.
          </Text>
        )}
      </Row>

      <Row justify="flex-end" mt={1}>
        <Button ml={2} onClick={handleSubmit} primary size="medium">
          Submit revision
        </Button>
      </Row>
    </Root>
  </ContextualBox>
)

const Root = styled.div`
  background-color: ${th('colorBackgroundHue2')};
  padding: calc(${th('gridUnit')} * 2);
`

export default reduxForm({ form: 'revision', destroyOnUnmount: false })(
  SubmitRevision,
)
