const config = require('config')
const Email = require('@pubsweet/component-email-templating')

const {
  User,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')
const { getEmailCopy } = require('./emailCopy')

const { name: journalName, staffEmail } = config.get('journal')

module.exports = {
  async sendNotifications({
    agree,
    baseUrl,
    collection,
    comments = '',
    User: UserModel,
    Fragment: FragmentModel,
  }) {
    const fragment = await FragmentModel.find(collection.fragments[0])
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })

    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const titleText = `manuscript titled "${parsedFragment.title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`
    const subjectBaseText = `${collection.customId}: Manuscript`
    const { customId } = collection
    const userHelper = new User({ UserModel })
    const subject = `${subjectBaseText} ${
      agree ? '' : 'Not '
    }Passed Technical Checks`

    const email = new Email({
      type: 'user',
      fromEmail: `${journalName} <${staffEmail}>`,
      content: {
        subject,
        signatureName: 'EQA Team',
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
        unsubscribeLink: baseUrl,
      },
    })

    sendEditorsEmail({
      email,
      agree,
      customId,
      comments,
      titleText,
      userHelper,
    })
  },
}

const sendEditorsEmail = async ({
  email,
  agree,
  titleText,
  customId,
  userHelper,
  comments = '',
}) => {
  email.content.subject = `${customId}: New manuscript submitted`
  const emailType = agree
    ? 'eqs-manuscript-accepted'
    : 'eqa-manuscript-returned-to-eic'

  const editors = (await userHelper.getEditorsInChief()).map(eic => ({
    ...eic,
    ...getEmailCopy({
      emailType,
      titleText,
      comments,
    }),
  }))

  editors.forEach(eic => {
    email.toUser = {
      email: eic.email,
      name: `${eic.firstName} ${eic.lastName}`,
    }
    const { html, text } = email.getNotificationBody({
      emailBodyProps: {
        paragraph: eic.paragraph,
        hasLink: eic.hasLink,
        hasIntro: eic.hasIntro,
        hasSignature: eic.hasSignature,
      },
    })
    email.sendEmail({ html, text })
  })
}
